@@ -1,6 +1,6 @@
 ;;; helm-bookmark.el --- Helm for Emacs regular Bookmarks. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2018 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -99,10 +99,11 @@
 (defvar helm-bookmark-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-map)
-    (define-key map (kbd "C-c o") 'helm-bookmark-run-jump-other-window)
-    (define-key map (kbd "C-d")   'helm-bookmark-run-delete)
-    (define-key map (kbd "C-]")   'helm-bookmark-toggle-filename)
-    (define-key map (kbd "M-e")   'helm-bookmark-run-edit)
+    (define-key map (kbd "C-c o")   'helm-bookmark-run-jump-other-window)
+    (define-key map (kbd "C-c C-o") 'helm-bookmark-run-jump-other-frame)
+    (define-key map (kbd "C-d")     'helm-bookmark-run-delete)
+    (define-key map (kbd "C-]")     'helm-bookmark-toggle-filename)
+    (define-key map (kbd "M-e")     'helm-bookmark-run-edit)
     map)
   "Generic Keymap for emacs bookmark sources.")
 
@@ -156,12 +157,19 @@
 (put 'helm-bookmark-toggle-filename 'helm-only t)
 
 (defun helm-bookmark-jump (candidate)
-  "Jump to bookmark from keyboard."
+  "Jump to bookmark action."
   (let ((current-prefix-arg helm-current-prefix-arg)
         non-essential)
     (bookmark-jump candidate)))
 
+(defun helm-bookmark-jump-other-frame (candidate)
+  "Jump to bookmark in other frame action."
+  (let ((current-prefix-arg helm-current-prefix-arg)
+        non-essential)
+    (bookmark-jump candidate 'switch-to-buffer-other-frame)))
+
 (defun helm-bookmark-jump-other-window (candidate)
+  "Jump to bookmark in other window action."
   (let (non-essential)
     (bookmark-jump-other-window candidate)))
 
@@ -237,7 +245,7 @@ BOOKMARK is a bookmark name or a bookmark record."
 BOOKMARK is a bookmark name or a bookmark record.
 This excludes bookmarks of a more specific kind (Info, Gnus, and W3m)."
   (let* ((filename   (bookmark-get-filename bookmark))
-         (isnonfile  (equal filename helm-bookmark--non-file-filename))) 
+         (isnonfile  (equal filename helm-bookmark--non-file-filename)))
     (and filename (not isnonfile) (not (bookmark-get-handler bookmark)))))
 
 (defun helm-bookmark-org-file-p (bookmark)
@@ -443,7 +451,6 @@ than `w3m-browse-url' use it."
 (defvar helm-bookmark-find-files-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-bookmark-map)
-    (define-key map (kbd "C-c o")   'ignore)
     (define-key map (kbd "C-x C-d") 'helm-bookmark-run-browse-project)
     map))
 
@@ -454,8 +461,10 @@ than `w3m-browse-url' use it."
   (call-next-method)
   (setf (slot-value source 'action)
         (helm-append-at-nth
-         (remove '("Jump to BM other window" . helm-bookmark-jump-other-window)
-                 helm-type-bookmark-actions)
+         (cl-loop for (name . action) in helm-type-bookmark-actions
+                  unless (memq action '(helm-bookmark-jump-other-frame
+                                        helm-bookmark-jump-other-window))
+                  collect (cons name action))
          '(("Browse project" . helm-bookmark-browse-project)) 1))
   (setf (slot-value source 'keymap) helm-bookmark-find-files-map))
 
@@ -661,6 +670,13 @@ words from the buffer into the new bookmark name."
 (put 'helm-bookmark-run-edit 'helm-only t)
 
 
+(defun helm-bookmark-run-jump-other-frame ()
+  "Jump to bookmark other frame from keyboard."
+  (interactive)
+  (with-helm-alive-p
+    (helm-exit-and-execute-action 'helm-bookmark-jump-other-frame)))
+(put 'helm-bookmark-run-jump-other-frame 'helm-only t)
+
 (defun helm-bookmark-run-jump-other-window ()
   "Jump to bookmark from keyboard."
   (interactive)
