@@ -59,13 +59,16 @@
                       minibuffer-local-must-match-filename-map ; Older Emacsen
                       minibuffer-local-must-match-map
                       minibuffer-local-ns-map))
-      (when (and (boundp map) (keymapp (symbol-value map)))
-        (when (and (boundp var) (symbol-value var))
-          (define-key (symbol-value map)
-              (read-kbd-macro (symbol-value var)) nil))
-        (when key
-          (define-key (symbol-value map)
-              (read-kbd-macro key) 'helm-minibuffer-history))))
+      (let ((vmap (symbol-value map)))
+        (when (and (boundp map) (keymapp vmap))
+          (helm-aif (and (boundp var) (symbol-value var))
+            (define-key vmap
+              (if (stringp it) (read-kbd-macro it) it)
+              nil))
+          (when key
+            (define-key (symbol-value map)
+              (if (stringp key) (read-kbd-macro key) key)
+              'helm-minibuffer-history)))))
     (set var key)))
 
 ;;; Command Keymap
