@@ -1,9 +1,9 @@
 ;;; helm-core-pkg.el --- define helm-core for package.el
 
-(define-package "helm-core" "3.0"
+(define-package "helm-core" "3.6.0"
   "Development files for Helm"
   '((emacs "24.4")
-    (async "1.9.3"))
+    (async "1.9.4"))
   :url "https://emacs-helm.github.io/helm/")
 
 ;; Local Variables:
